/*
The programm for gas detector

*/

	
#include "map.h"            // processor register defines
#include "sci.h"            // Bootloader's SCI included


#define BYTE unsigned char
#define WORD unsigned int

#define EnableInterrupts()  asm CLI
#define DisableInterrupts() asm SEI


#define Gas 2
#define Reference 3
#define Consumption 0			// number of ADC input for given value

#define Alarm2Level 20				// delta between alarm1 and alarm2 levels
#define Hysteresis 5				// hysteresis on measurement
#define CurrentLimit 252				// voltage level for sensor current problem
#define WaitTime 2					// how long time I need signal before alarm
#define SensKOLow 10				// below this value the sensor is bad


/////////////////////////////////////////////////////
// I/O mappings

#define LED    PTA1
#define OUTPUT 	PTA3
#define MEMORY_FUN 	PTA2

/////////////////////////////////////////////////////


#pragma CONST_SEG CONST_ROM

#pragma CONST_SEG DEFAULT


#pragma DATA_SEG SHORT MY_ZEROPAGE

WORD Tick1;
BYTE Tick2,Led_on;
unsigned int x;
unsigned int i;

#pragma DATA_SEG DEFAULT

BYTE ConsumptionValue, GasValue;
BYTE Alarm1, Alarm2, SensorAlarm;
BYTE ReferenceValue;
BYTE Al_enable;

void Init(void)
{
#ifdef COP_ENABLE
    CONFIG1 = 0x00;
#else
    CONFIG1 = CONFIG1_COPD;
#endif
    
    DDRA = 0xFF;			// all outputs
    DDRA_BIT0 = 0;			// PTA0 is input
    DDRA_BIT4 = 0;			// PTA4 is input
    DDRA_BIT5 = 0;			// PTA5 is input
    PTAPUE = 0x00;			// no pull ups
    
    TSC_PS2 = 1;
    TSC_PS1 = 1;
    TSC_PS0 = 0;			// prescaller /64
      
	TSC_TOIE = 1;			// enables TIM interrupt
	TSC_TSTOP = 0;			// runs timer

    TMODH = 0xC3;				
    TMODL = 0x50;			// 1sec preset

    EnableInterrupts();
    
    
}

interrupt IV_IRQ1 void Int_IRQ(void)
{
//    ISCR_ACK1 = 1;              /* IRQ ack */
}

interrupt IV_TCHO void Int_TimerCH0(void)
{
//    TSC0;
//    TSC0_CH0F = 0;	        /* clearing CH0 flag */

}

interrupt IV_TCH1 void Int_TimerCH1(void)
{
//    TSC1;
//    TSC1_CH1F = 0;	        /* clearing CH1 flag */
}

interrupt IV_TOVF void Int_TimerTOV(void)
{
    TSC;
    TSC_TOF = 0;              /* clearing TOV flag */
  	TMOD = 500;				//10ms
	Tick1++;
	Tick2++;

	if (Al_enable == 1)
	{
		if (Tick2 < 2) 
		{
			LED = 0;		//LED is on
			OUTPUT = 1;		
		}
		else 
		{
			LED = 1;
			OUTPUT = 0;		
		}
		if (Tick2 > Led_on) Tick2 = 0;
	}
}

interrupt IV_KBRD void Int_Keyboard(void)
{
//    KBSCR_ACKK = 1;         /* keyb int. acknowledge */

}

interrupt IV_ADC void Int_ADConv(void)
{
BYTE temp;

//    temp = ADR;             /* just read ADR or write to ADSCR to clear int. */

}


/*-------------------------------------------------------*/

void Waiting(unsigned int Koliko)
{

	Tick1 = 0;
	while(Tick1 != (Koliko*100));


}


/*-----------------------------------------------------*/

BYTE Measure (BYTE InputNumber)
{

BYTE temp;

	ADSCR = 0x80 | InputNumber;					// sets one conversion, no interrupt
	while(ADSCR_COCO == 0);							// waits for conversion complete
	temp = ADR;									// reads value
	return(temp);

}

/*-----------------------------------------------------*/
void main(void) 
{
    Init();
    SCIAPIInit();


#ifdef COP_ENABLE
        COPCTL = 0;             /* bump watchdog */
#endif

	LED = 0;							// switch led on
	Waiting (150);						// waits 150sec after reset - sensor needs it
	SensorAlarm = 0;

	for (;;)								// stadard operation, wait for Ticks and then measure&write
	{
		GasValue = Measure(Gas);
		ReferenceValue = Measure(Reference);
		ConsumptionValue = Measure(Consumption);

		if (GasValue < (ReferenceValue - Hysteresis))
		{
			Alarm1 = 0;
			Alarm2 = 0;				// delay alarms, level is below treshold
		}
		if (GasValue > ReferenceValue)
		{
			Al_enable = 1;
			if (GasValue > (ReferenceValue + Hysteresis))
			{
				Alarm2 = 1;				// 2nd alarm level, big concentration
				Alarm1 = 0;
			}
			else 
			{
				Alarm2 = 0;
				Alarm1 = 1;					// alarm 1 level
			}
		}
		if (ConsumptionValue > CurrentLimit)
		{
			SensorAlarm	= 1;				// sensor alarm!
		}
		else if (GasValue < SensKOLow)
		{
			SensorAlarm	= 1;				// sensor alarm!
		}
		else SensorAlarm = 0;
		
		if ((SensorAlarm == 0) & ((Alarm1 == 0) & (Alarm2 == 0)))
		{
			if (MEMORY_FUN)				// if mem_fun pin is open, then delete alarms
			{
				Tick1 = 0;				// no alarm, clear flags
				LED = 1;	
				OUTPUT = 0;
				Al_enable = 0;
			}
		}
		
		else if (Tick1 >= WaitTime)
		{
			if (Alarm1)	Led_on = 25;
			if (Alarm2)	Led_on = 10;
			if (SensorAlarm) Led_on = 1;
		}
		
		Waiting (1);

	}

}